#include <iostream>

#include <Executor.h>

using namespace std;

Object Executor::visitAssign(ExprLabeledParser::AssignContext *ctx)
{
    string idName  = ctx->IDENTIFIER()->getText();
    Object value   = visit(ctx->expr());  // compute the expression value
    symtab[idName] = value;               // store it into the symbol table

    return value;
}

Object Executor::visitPrint(ExprLabeledParser::PrintContext *ctx)
{
    Object value = visit(ctx->expr());  // evaluate the expression
    cout << value.as<int>() << endl;    // print the result

    return Object();
}

Object Executor::visitInt(ExprLabeledParser::IntContext *ctx)
{
    return stoi(ctx->INTEGER()->getText());  // integer value
}

Object Executor::visitId(ExprLabeledParser::IdContext *ctx)
{
    string id = ctx->IDENTIFIER()->getText();

    if (symtab.find(id) != symtab.end())
    {
        return symtab[id];  // value from symbol table
    }
    else return 0;          // dummy value
}

Object Executor::visitMulDiv(ExprLabeledParser::MulDivContext *ctx)
{
    int left  = visit(ctx->expr(0)).as<int>();  // left  child expression
    int right = visit(ctx->expr(1)).as<int>();  // right child expression

    if (ctx->op->getType() == ExprLabeledParser::MUL) return left*right;
    else                                              return left/right;
}

Object Executor::visitAddSub(ExprLabeledParser::AddSubContext *ctx)
{
    int left  = visit(ctx->expr(0)).as<int>();  // left  child expression
    int right = visit(ctx->expr(1)).as<int>();  // right child expression

    if (ctx->op->getType() == ExprLabeledParser::ADD) return left + right;
    else                                              return left - right;
}

Object Executor::visitParens(ExprLabeledParser::ParensContext *ctx)
{
    return visit(ctx->expr());  // return parenthesized expression value
}
